# Copyright 2020 Unlimited Robotics
import re
import tempfile
import platform
import subprocess
from pathlib import Path

RAYAENV_DOCKER_VERSION_DEFAULT = '0.0.0'
GARYSIM_VERSION_DEFAULT = '0.0.0'
GARYSIM_VERSION = GARYSIM_VERSION_DEFAULT
RAYAENV_DOCKER_VERSION = RAYAENV_DOCKER_VERSION_DEFAULT

# Json SCAN Keys
JSON_SCAN_ID = 'ROBOT_ID'
JSON_SCAN_IP = 'ROBOT_IP'
JSON_SCAN_SERIAL = 'ROBOT_SERIAL'

# ssh ROBOT
SSH_USER = 'rayadevel'
SSH_PORT = '2222'
SSH_APP_FOLDER = '~/apps/'

# Json SDK Info Keys
JSON_EXECINFO_APPID = 'app-id'
JSON_EXECINFO_APPNAME = 'app-name'
JSON_EXECINFO_SIM = 'simulation'
JSON_EXECINFO_ROBCONN = 'robot-connection'
JSON_EXECINFO_ROBID = 'robot-id'
JSON_EXECINFO_ROBIP = 'ip-address'
JSON_EXECINFO_ROBSERIAL = 'robot-serial'

# Folders
TEMP_URSDK_FOLDER = 'ursdk'
TEMPLATES_FOLDER = 'template'
DAT_FOLDER = 'dat'
DOC_FOLDER = 'doc'
LOG_FOLDER = 'log'
RES_FOLDER = 'res'
SRC_FOLDER = 'src'
VSCODE_FOLDER = '.vscode'

# Files
LAST_SCANNING_FILE = 'last_scanning.json'
ENTRYPOINT_FILE = '__main__.py'
VSCODE_SETTINGS_FILE = 'launch.json'
APP_FILE = 'app.py'
MANIFEST_FILE = 'manifest.json'
EXECSETTINGS_FILE = 'exec_settings.json'
CONNECTION_SETTINGS_FILE = 'connection.json'

# System paths
TEMP_PATH = Path(tempfile.gettempdir())
CURRENT_PATH = Path().absolute()
URSDK_PATH = Path(__file__).parent.absolute()
UR_HOME = Path.home() / '.ur'
SIMS_HOME = UR_HOME / 'simulator'
SIMS_APPS = SIMS_HOME / 'apps'
SIMS_DATA_APPS = SIMS_HOME / 'data_apps'
SIMS_DATA_APPS_DEVEL = SIMS_HOME / 'data_apps_devel'

# Derived folder paths
URSDK_TEMP_PATH = TEMP_PATH / TEMP_URSDK_FOLDER
TEMPLATES_PATH = URSDK_PATH / TEMPLATES_FOLDER
DAT_PATH = CURRENT_PATH / DAT_FOLDER
DOC_PATH = CURRENT_PATH / DOC_FOLDER
LOG_PATH = CURRENT_PATH / LOG_FOLDER
RES_PATH = CURRENT_PATH / RES_FOLDER
SRC_PATH = CURRENT_PATH / SRC_FOLDER
VSCODE_PATH = CURRENT_PATH / VSCODE_FOLDER

# derived files paths
LAST_SCANNING_PATH = URSDK_TEMP_PATH / LAST_SCANNING_FILE
ENTRYPOINT_PATH_ORIG = TEMPLATES_PATH / ENTRYPOINT_FILE
ENTRYPOINT_PATH_DEST = CURRENT_PATH / ENTRYPOINT_FILE
VSCODE_SETTINGS_PATH_ORIG = TEMPLATES_PATH / VSCODE_SETTINGS_FILE
VSCODE_SETTINGS_PATH_DEST = VSCODE_PATH / VSCODE_SETTINGS_FILE
APP_PATH_ORIG = TEMPLATES_PATH / APP_FILE
APP_PATH_DEST = SRC_PATH / APP_FILE
MANIFEST_PATH_ORIG = TEMPLATES_PATH / MANIFEST_FILE
MANIFEST_PATH_DEST = CURRENT_PATH / MANIFEST_FILE
EXECSETTINGS_PATH = CURRENT_PATH / EXECSETTINGS_FILE
CONNECTION_SETTINGS_PATH = UR_HOME / CONNECTION_SETTINGS_FILE

# Docker Environment
RAYAENV_DOCKER_UR_ROOT = SIMS_HOME / 'ur_root'
RAYAENV_DOCKER_UR_ROOT_REPO = 'git clone --branch garyunity git@github.com:Unlimited-Robotics/raya_root_folder_templates.git'
RAYAENV_DOCKER_IMGNAME = 'raya_os'
RAYAENV_DOCKER_IMGNAME_DEV = 'raya_sim_dev'
RAYAENV_DOCKER_IMG_VERSION_NAME = 'sim'
RAYAENV_DOCKER_IMG_VERSION_GPU = 'sim_gpu'


def check_if_gpu():
    global RAYAENV_DOCKER_IMAGE_NAME
    return_code = subprocess.call(
        'docker info | grep "Runtimes:" | grep "nvidia"',
        shell=True,
        stdout=subprocess.PIPE)
    if return_code != 0:
        RAYAENV_DOCKER_IMAGE_NAME = f'unlimitedrobotics/{RAYAENV_DOCKER_IMGNAME}.{RAYAENV_DOCKER_IMG_VERSION_NAME}.prod'
    else:
        RAYAENV_DOCKER_IMAGE_NAME = f'unlimitedrobotics/{RAYAENV_DOCKER_IMGNAME}.{RAYAENV_DOCKER_IMG_VERSION_GPU}.prod'


def read_local_version_file():
    global RAYAENV_DOCKER_VERSION, GARYSIM_VERSION
    try:
        with open(f'{UR_HOME}/versions.txt', 'r') as f:
            for line in f:
                if re.match(r'Docker:\d.\d.\d', line):
                    RAYAENV_DOCKER_VERSION = re.findall(
                        r':(.*?)(?:\r?\n|$)', line)[0]
                elif re.match(r'Unity:\d.\d.\d', line):
                    GARYSIM_VERSION = re.findall(r':(.*?)(?:\r?\n|$)', line)[0]
    except FileNotFoundError:
        UR_HOME.mkdir(parents=True, exist_ok=True)
        with open(f'{UR_HOME}/versions.txt', 'w+') as f:
            f.write((f'Docker:{RAYAENV_DOCKER_VERSION_DEFAULT}\n'
                     f'Unity:{GARYSIM_VERSION_DEFAULT}'))
            f.close()


# Gary Simulator
def gary_sim():
    global SIM_BINARY, GARYSIM_URL, SIM_TARPATH, GARYSIM_SHA256, SIMS_HOME

    if platform.system() == 'Linux':
        # TODO: Change the dev release to the oficial
        GARYSIM_URL = f'https://storage.googleapis.com/raya_files/Linux/raya_simulator_{GARYSIM_VERSION}_linux.tar.gz'
        GARYSIM_SHA256 = 'ebb28416546d90cbdcce0813f988eb2ce5e9c90fc4e28a52e53b297ab4744df8'

        SIM_PATH = SIMS_HOME / f'raya_simulator_{GARYSIM_VERSION}_linux'
        SIM_TARPATH = SIMS_HOME / f'raya_simulator_{GARYSIM_VERSION}_linux.tar.gz'
        SIM_BINARY = SIM_PATH / 'Linux.x86_64'

    elif platform.system() == 'Windows':
        GARYSIM_URL = f'https://storage.googleapis.com/raya_files/Windows/raya_simulator_{GARYSIM_VERSION}_windows.tar.gz'
        GARYSIM_SHA256 = '3d3f0a68f29b0d8efe6fc24c7fe8dd5ca08d7ec9bf74aa58f9826e4c4a67d0e8'

        SIM_PATH = SIMS_HOME / f'raya_simulator_{GARYSIM_VERSION}_windows'
        SIM_TARPATH = SIMS_HOME / f'raya_simulator_{GARYSIM_VERSION}_windows.tar.gz'
        SIM_BINARY = SIM_PATH / 'Windows.exe'

    # TODO: MAC version - M1
    elif platform.system() == 'Darwin':
        GARYSIM_URL = f'https://storage.googleapis.com/raya_files/Mac/raya_simulator_{GARYSIM_VERSION}_mac.tar.gz'
        GARYSIM_SHA256 = '43f0b39f540079f5446e383d1cc3304084de95dc52c5bd5dd7947eaaa175ba1c'

        SIM_PATH = SIMS_HOME / f'raya_simulator_{GARYSIM_VERSION}_mac'
        SIM_TARPATH = SIMS_HOME / f'raya_simulator_{GARYSIM_VERSION}_mac.tar.gz'
        SIM_BINARY = SIM_PATH / 'Mac.App'


# Method for updating constants
def update_constants():
    read_local_version_file()
    check_if_gpu()
    gary_sim()


update_constants()

# VCS (Version Control System)
RAYA_OS_COMPONENTS = ['SDK', 'Unity', 'Docker']
#VCS_URL = 'https://dev.robotics.dev.webiz.ge/vcs'
# url = f'{VCS_URL}/version?major={self.__get_major(version)}&medium={self.__get_minor(version)}&platform={self.host_os}'
VCS_URL = 'https://raw.githubusercontent.com/Unlimited-Robotics/dirty_vcs/prod/vcs_info.json'
SYSTEM_DICT = {'Linux': 'linux', 'Windows': 'windows', 'Darwin': 'mac'}
