# Copyright 2020 Unlimited Robotics

import os
import json
from shutil import copyfile

from rayasdk.logger import log_error, log_verbose, log_info
from rayasdk.constants import *
from rayasdk.utils import *


class URInitializer:
    COMMAND = 'init'

    def __init__(self):
        pass

    def init_parser(self, subparser):
        self.parser = subparser.add_parser(
            type(self).COMMAND,
            help="initialize Raya project in current folder.")
        self.parser.add_argument('--app-id',
                                 help='application unique identificator',
                                 required=True,
                                 type=str)
        self.parser.add_argument('--app-name',
                                 help='application name',
                                 type=str)

    def create_files_tree(self):
        try:
            os.mkdir(DOC_PATH)
            os.mkdir(LOG_PATH)
            os.mkdir(RES_PATH)
            os.mkdir(SRC_PATH)
            os.mkdir(VSCODE_PATH)
            copyfile(APP_PATH_ORIG, APP_PATH_DEST)
            copyfile(MANIFEST_PATH_ORIG, MANIFEST_PATH_DEST)
            copyfile(ENTRYPOINT_PATH_ORIG, ENTRYPOINT_PATH_DEST)

            copyfile(VSCODE_SETTINGS_PATH_ORIG, VSCODE_SETTINGS_PATH_DEST)
        except OSError as exc:
            print(exc)
            raise Exception(f'Could not create files tree')
        return True

    def create_json_exec_info(self):
        exec_settings = {}
        exec_settings[JSON_EXECINFO_APPID] = self.args.app_id
        exec_settings[JSON_EXECINFO_APPNAME] = self.args.app_name
        try:
            with open(EXECSETTINGS_PATH, 'w', encoding='utf-8') as f:
                json.dump(exec_settings, f, ensure_ascii=False, indent=4)
        except FileNotFoundError:
            log_error(f'Could not write file "{EXECSETTINGS_FILE}".')
            return False
        return True

    def run(self, args, unknownargs):
        self.args = args
        self.unknownargs = unknownargs

        # Check if the parent has the same name that the app id
        validate_app_id_and_folder_name(self.args.app_id)

        # Check if the directory is empty
        if len(os.listdir(CURRENT_PATH)) != 0:
            raise Exception('Current directory is not empty.')

        # Files tree
        log_verbose('Creating application files tree...')
        self.create_files_tree()

        # Json SDK info
        log_verbose('Creating exec file...')
        self.create_json_exec_info()

        log_info('\nCorrect!')
        return True
