from enum import Enum

verbose = False
quiet = False


class bcolors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKCYAN = '\033[96m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'


class LogLevels(Enum):
    VERBOSE = 1
    INFO = 2
    WARNING = 3
    ERROR = 4


def set_logger_level(_verbose, _quiet):
    global verbose, quiet
    verbose = _verbose
    quiet = _quiet


def log(level, msg, end='\n', flush=False):
    if level == LogLevels.INFO or (level == LogLevels.VERBOSE and verbose):
        if not quiet:
            print(msg, end=end, flush=flush)
    elif level == LogLevels.ERROR:
        print(msg, end=end, flush=flush)
    elif level == LogLevels.WARNING:
        print(msg, end=end, flush=flush)


def log_verbose(msg, end='\n', flush=False):
    log(LogLevels.VERBOSE, msg, end=end, flush=flush)


def log_info(msg, end='\n', flush=False):
    log(LogLevels.INFO, msg, end=end, flush=flush)


def log_warning(msg, end='\n', flush=False):
    log(LogLevels.WARNING,
        f'{bcolors.WARNING}WARNING:{bcolors.ENDC} {msg}',
        end=end,
        flush=flush)


def log_error(msg, end='\n', flush=False):
    log(LogLevels.ERROR,
        f'{bcolors.FAIL}ERROR:{bcolors.ENDC} {msg}',
        end=end,
        flush=flush)
