# Copyright 2020 Unlimited Robotics
import os
from time import sleep

from rayasdk.utils import run_simulator_bridge
from rayasdk.constants import *
from rayasdk.logger import *


class URSimulator:

    COMMAND = 'simulator'

    def __init__(self):
        pass

    def init_parser(self, subparser):
        self.parser = subparser.add_parser(type(self).COMMAND,
                                           help="run the simulator bridge.")
        self.parser.add_argument(
            '--debug',
            help='Runs only the unity simulation for dev on the container',
            action="store_true")

    def run(self, args, unknownargs):
        self.args = args
        self.unknownargs = unknownargs

        if not os.path.exists(SIM_BINARY):
            log_error('Simulator not found, run rayasdk update to download it')
            return False

        # If debug flag not set run container
        container_process = None
        if not self.args.debug:
            try:
                container_process = run_simulator_bridge()
                if type(container_process) == bool:
                    return False
                sleep(5)
            except KeyboardInterrupt:
                pass

        # Run unity simulation
        simulator_process = subprocess.Popen([str(SIM_BINARY)] +
                                             self.unknownargs,
                                             stdout=subprocess.PIPE)

        if container_process is not None:
            while True:
                if container_process.poll() is not None:
                    break
            container_process.terminate()
        else:
            simulator_process.wait()

        simulator_process.terminate()
        return True
